<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Domain\Model;

use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use IMIA\BaseExt\Annotation\SQL;
use IMIA\BaseExt\Annotation\TCA;

/**
 * @package     imia_pageteaser
 * @subpackage  Domain\Model
 * @author      David Frerich <d.frerich@imia.de>
 * )
 */
abstract class AbstractPage extends \IMIA\BaseExt\Domain\Model\Page
{
    /**
     * @var \TYPO3\CMS\Core\Resource\FileRepository
     * @inject
     * @FIXME How to do this a better way?
     */
    protected $fileRepository;

    /**
     * @var IMIA\ImiaPageteaser\Domain\Repository\PageRepository
     * @inject
     * @FIXME How to do this a better way to sort the relation-items with a 1:n field?
     */
    protected $pageRepository;

    /**
     * @var string
     */
    protected $teaserPages;

    /**
     * @var array
     */
    protected $teaserPageObjects;

    /**
     * @var integer
     */
    protected $teaserImage;

    /**
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Category>
     */
    protected $categories;

    /**
     * @var DateTime
     */
    protected $lastUpdated;

    /**
     * @var integer
     */
    protected $contentFileRelations;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->categories = new ObjectStorage();
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\FileReference
     * @FIXME How to do this a better way?
     */
    public function getTeaserImage()
    {
        if (!$this->teaserImage || !is_object($this->teaserImage)) {
            $files = $this->fileRepository->findByRelation('pages', 'teaser_image', $this->getUid());
            $this->teaserImage = array_shift($files);
        }

        return $this->teaserImage;
    }

    /**
     * @return integer
     * @FIXME Very dirty, but is there a better way?
     */
    public function getContentFileRelations()
    {
        if (!$this->contentFileRelations) {
            $this->contentFileRelations = (int)@array_shift(
                $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow(
                    'count(*) as count',
                    'sys_file_reference',
                    'tablenames="tt_content"' .
                        ' AND pid=' . $this->getUid() .
                        ' AND deleted=0' .
                        ' AND hidden=0'
                )
            );
        }

        return $this->contentFileRelations;
    }

    /**
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Category>
     */
    public function getCategories()
    {
        return $this->categories;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Category
     */
    public function getCategory()
    {
        $this->getCategories()->rewind();

        return $this->getCategories()->current();
    }

    /*
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\IMIA\ImiaPageteaser\Domain\Model\Page>
     */
    public function getTeaserPages()
    {
        if (!$this->teaserPageObjects) {
            $teaserPageUids = array_map('intval', explode(',', $this->teaserPages));
            $teaserPages = $this->pageRepository->findByUids($teaserPageUids);

            $teaserPageObjects = array();
            foreach($teaserPages as $teaserPage) {
                $teaserPageObjects[$teaserPage->getUid()] = $teaserPage;
            }

            $this->teaserPageObjects = array();
            foreach ($teaserPageUids as $teaserPageUid) {
                if (array_key_exists($teaserPageUid, $teaserPageObjects)) {
                    $this->teaserPageObjects[] = $teaserPageObjects[$teaserPageUid];
                }
            }
        }

        return $this->teaserPageObjects;
    }
}