<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Domain\Repository;

/**
 * @package     imia_pageteaser
 * @subpackage  Domain\Repository
 * @author      David Frerich <d.frerich@imia.de>
 */
class PageRepository extends \IMIA\BaseExt\Domain\Repository\NoStorageRepository
{
    /**
     * @var \IMIA\ImiaPageteaser\Domain\Repository\PageAltRepository
     * @inject
     * @FIXME How to do this a better way?
     */
    public $pageAltRepository;

    /**
     * @param array $uids
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByUids(array $uids, $limit = null, $offset = null, $sorting = null)
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $query->matching(
            $query->in('uid', $uids)
        );

        return $this->executeQuery($query);
    }

    /**
     * @param array $uids
     */
    public function findTeaserPagesByUids(array $uids)
    {
        $query = $this->createQuery();
        $query->matching(
            $query->in('uid', $uids)
        );

        $uids = array();
        $result = $this->executeArrayQuery($query, array('teaser_pages'));
        foreach ($result as $row) {
            $uids = array_merge($uids, array_map('intval', explode(',', $row['teaser_pages'])));
        }

        return array_unique($uids);
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return array
     */
    public function findUidsByDoktypesAndCategories(array $doktypes, array $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter, null, null, $sorting);

        return $this->executeUidArrayQuery($query);
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findUidsByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, array $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter, null, null, $sorting);

        return $this->executeUidArrayQuery($query);
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findUidsByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, array $categories = null,
        array $filter = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter, null, null, $sorting);

        return $this->executeUidArrayQuery($query);
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @return integer
     */
    public function countByDoktypesAndCategories(array $doktypes, array $categories = null, array $filter = null)
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @return integer
     */
    public function countByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, array $categories = null, array $filter = null)
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @return integer
     */
    public function countByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, array $categories = null, array $filter = null)
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter);

        return $query->count();
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByDoktypesAndCategories(array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByDoktypesAndCategories($doktypes, $categories, $filter, $limit, $offset, $sorting);

        return $this->executeQuery($query);
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByPidsAndDoktypesAndCategories(
            $this->getResolvedPageUids($pIds), $doktypes, $categories, $filter, $limit, $offset, $sorting);

        return $this->executeQuery($query);
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQueryByUidsAndDoktypesAndCategories($uIds, $doktypes, $categories, $filter, $limit, $offset, $sorting);

        return $this->executeQuery($query);
    }

    /**
     * @param array $pIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\Query
     */
    public function createQueryByPidsAndDoktypesAndCategories(array $pIds, array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $constraints = array(
            $query->in('pid', $pIds),
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param array $uIds
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\Query
     */
    public function createQueryByUidsAndDoktypesAndCategories(array $uIds, array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $constraints = array(
            $query->in('uid', $uIds),
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param array $doktypes
     * @param array $categories
     * @param array $filter
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return \TYPO3\CMS\Extbase\Persistence\Query
     */
    protected function createQueryByDoktypesAndCategories(array $doktypes, array $categories = null,
        array $filter = null, $limit = null, $offset = null, $sorting = array('sorting', 'asc'))
    {
        $query = $this->createQuery($limit, $offset, $sorting);

        $constraints = array(
            $query->in('doktype', $doktypes)
        );
        $this->addCategoriesConstraints($query, $constraints, $categories);
        $this->addFilterConstraints($query, $constraints, $filter);

        $query->matching(
            $query->logicalAnd($constraints)
        );

        return $query;
    }

    /**
     * @param type $pageUids
     * @return array
     */
    protected function getResolvedPageUids(array $pageUids)
    {
        $query = $this->createQuery();
        $query->matching($query->in('uid', $pageUids));

        $resolvedPageUids = array();
        foreach ($query->execute() as $page) {
            if ($page->getDoktype() == 7 && $page->getMountPid()) {
                $resolvedPageUids[] = $page->getMountPid();
            } elseif ($page->getContentFromPid()) {
                $resolvedPageUids[] = $page->getContentFromPid();
            } else {
                $resolvedPageUids[] = $page->getUid();
            }
        }

        return $resolvedPageUids;
    }

    /**
     * @param object $query
     * @param object $constraints
     * @param array $categories
     */
    protected function addCategoriesConstraints($query, array &$constraints, array $categories = null)
    {
        if ($categories && count($categories) > 0) {
            $categoryConstraints = array();
            foreach ($categories as $category) {
                $categoryConstraints[] = $query->contains('categories', $category);
            }
            $constraints[] = $query->logicalOr($categoryConstraints);
        }
    }

    /**
     * @param object $query
     * @param object $constraints
     * @param array $filter
     */
    protected function addFilterConstraints($query, array &$constraints, array $filter = null)
    {
        if ($filter && count($filter) > 0) {
            if (array_key_exists('word', $filter) && $filter['word']) {
                $constraints[] = $query->logicalOr(
                    $query->like('title', '%' . $filter['word'] . '%'),
                    $query->like('abstract', '%' . $filter['word'] . '%')
                );
            }

            if (array_key_exists('year', $filter) && $filter['year']) {
                if (array_key_exists('month', $filter) && $filter['month']) {
                    if (array_key_exists('day', $filter) && $filter['day']) {
                        // year and month and day
                        $constraints[] = $query->logicalAnd(
                            $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], (int)$filter['day'], (int)$filter['year'])),
                            $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], (int)$filter['day']+1, (int)$filter['year']))
                        );
                    } else {
                        // year and month
                        $constraints[] = $query->logicalAnd(
                            $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], 1, (int)$filter['year'])),
                            $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month']+1, 1, (int)$filter['year']))
                        );
                    }
                } else {
                    // years
                    $constraints[] = $query->logicalAnd(
                        $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, 1, 1, (int)$filter['year'])),
                        $query->lessThan('lastUpdated', mktime(0, 0, 0, 13, 1, (int)$filter['year']))
                    );
                }
            } elseif (array_key_exists('month', $filter) && $filter['month']) {
                $monthConstraints = array();
                for ($i = 2000; $i <= (int)date('Y'); $i++) {
                    // month
                    $monthConstraints = $query->logicalAnd(
                        $query->greaterThanOrEqual('lastUpdated', mktime(0, 0, 0, (int)$filter['month'], 1, $i)),
                        $query->lessThan('lastUpdated', mktime(0, 0, 0, (int)$filter['month']+1, 1, $i))
                    );
                }
                $constraints[] = $query->logicalOr($monthConstraints);
            }

            if (array_key_exists('pid', $filter) && $filter['pid']) {
                $constraints[] = $query->equals('pid', (int)$filter['pid']);
            }
        }
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\Query $query
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    protected function executeQuery($query)
    {
        $pages = $query->execute();

        $pageUids = array();
        $pagesArray = array();
        foreach ($pages as $page) {
            $pageUids[] = $page->getUid();
            $pagesArray[$page->getUid()] = $page;
        }

        if ($GLOBALS['TSFE']->sys_language_uid > 0) {
            $altPages = $this->pageAltRepository->findByPagesAndLanguage($pageUids, $GLOBALS['TSFE']->sys_language_uid);
            foreach($altPages as $altPage) {
                $pagesArray[$altPage->getPage()->getUid()] = $altPage;
            }
        }
        $pages = $pagesArray;

        // l18n_cfg setting
        foreach ($pages as $key => $page) {
            switch ($page->getL18nCfg()) {
                case 1:
                    // hide default translation
                    if ($GLOBALS['TSFE']->sys_language_uid == 0) {
                        unset($pages[$key]);
                    }
                    break;
                case 2:
                    // hide page if no translation for current language exists
                    if ($GLOBALS['TSFE']->sys_language_uid != $page->getSysLanguageUid()) {
                        unset($pages[$key]);
                    }
                    break;
                case 3:
                    // hide default translation
                    // hide page if no translation for current language exists
                    if ($GLOBALS['TSFE']->sys_language_uid == 0
                      || $GLOBALS['TSFE']->sys_language_uid != $page->getSysLanguageUid()) {
                        unset($pages[$key]);
                    }
                    break;
                case 0:
                default:
                    break;
            }
        }

        return $pages;
    }
}